#include "DialogManagePlugins.h"
#include "ui_DialogManagePlugins.h"

enum {
    FIELD_IS_LOADED,
    FIELD_AUTO_LOAD,
    FIELD_PLUGIN_NAME
};

DialogManagePlugins::DialogManagePlugins(QNNTPGrabGlue *glue, QWidget *parent) :
QDialog(parent),
ui(new Ui::DialogManagePlugins)
{
    this->glue = glue;
    ui->setupUi(this);

    connect(glue, SIGNAL(pluginLoaded(QString,bool)), SLOT(onpluginLoaded(QString,bool)));
    connect(glue, SIGNAL(pluginUnloaded(QString)), SLOT(onpluginUnloaded(QString)));

    QStringList auto_load_plugins = gui_opts.getAutoLoadPlugins();
    QList<QString> plugins = this->glue->getAvailPlugins();
    bool first = true;
    int i ;
    for (i = 0; i < plugins.length(); i++) {
        QNNTPGrabGlue::PluginInfo plugin_info;
        if (!glue->getPluginInfo(plugins[i], plugin_info)) {
            qWarning("Unable to find a plugin named '%s'", plugins[i].toUtf8().data());
            continue;
        }

        QTreeWidgetItem *plugin_item = new QTreeWidgetItem();
        if (plugin_info.is_loaded) {
            plugin_item->setCheckState(FIELD_IS_LOADED, Qt::Checked);
        } else {
            plugin_item->setCheckState(FIELD_IS_LOADED, Qt::Unchecked);
        }

        if (auto_load_plugins.contains(plugins[i])) {
            plugin_item->setCheckState(FIELD_AUTO_LOAD, Qt::Checked);
        } else {
            plugin_item->setCheckState(FIELD_AUTO_LOAD, Qt::Unchecked);
        }
        plugin_item->setText(FIELD_PLUGIN_NAME, plugin_info.name);

        ui->treeWidget->addTopLevelItem(plugin_item);
        if (first) {
            ui->treeWidget->setItemSelected(plugin_item, true);
            first = false;
        }
    }

    connect(ui->treeWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)), SLOT(itemChanged(QTreeWidgetItem*,int)));
}

DialogManagePlugins::~DialogManagePlugins()
{
    delete ui;
}

void DialogManagePlugins::onpluginLoaded(QString plugin_name, bool is_persistent)
{
    Q_UNUSED(is_persistent);

    for (int i = 0; i < ui->treeWidget->topLevelItemCount(); i++) {
        QTreeWidgetItem *item = ui->treeWidget->topLevelItem(i);

        if (item->text(FIELD_PLUGIN_NAME) != plugin_name) {
            continue;
        }

        if (item->checkState(FIELD_IS_LOADED) == Qt::Unchecked) {
            item->setCheckState(FIELD_IS_LOADED, Qt::Checked);
        }
    }
}


void DialogManagePlugins::onpluginUnloaded(QString plugin_name)
{
    for (int i = 0; i < ui->treeWidget->topLevelItemCount(); i++) {
        QTreeWidgetItem *item = ui->treeWidget->topLevelItem(i);

        if (item->text(FIELD_PLUGIN_NAME) != plugin_name) {
            continue;
        }

        if (item->checkState(FIELD_IS_LOADED) == Qt::Checked) {
            item->setCheckState(FIELD_IS_LOADED, Qt::Unchecked);
        }
    }
}

void DialogManagePlugins::on_treeWidget_itemSelectionChanged()
{
    if(ui->treeWidget->selectedItems().count() != 1)
        return;

    QTreeWidgetItem *item = ui->treeWidget->selectedItems()[0];
    QNNTPGrabGlue::PluginInfo plugin_info;
    if(!glue->getPluginInfo(item->text(FIELD_PLUGIN_NAME), plugin_info)) {
        qWarning("Unable to get plugin info for plugin: %s",item->text(FIELD_PLUGIN_NAME).toUtf8().data());
        return;
    }

    ui->lb_plugin_name->setText(plugin_info.name);
    ui->lb_plugin_author->setText(plugin_info.author);
    ui->lb_plugin_version->setText(plugin_info.version);
    ui->lb_plugin_url->setText(plugin_info.url);
    ui->lb_plugin_description->setText(plugin_info.description);
}

void DialogManagePlugins::itemChanged(QTreeWidgetItem *item, int column)
{
    QString errmsg;
    QString plugin_name = item->text(FIELD_PLUGIN_NAME);
    QNNTPGrabGlue::PluginInfo plugin_info;

    if (column == FIELD_IS_LOADED) {
        Q_ASSERT(glue->getPluginInfo(plugin_name, plugin_info));

        if (plugin_info.is_loaded && item->checkState(FIELD_IS_LOADED) == Qt::Unchecked) {
            if (!glue->unloadPlugin(plugin_name, errmsg)) {
                qWarning("%s", (tr("Unable to unload plugin '%1': %2").arg(plugin_name).arg(errmsg)).toUtf8().data());
                item->setCheckState(FIELD_IS_LOADED, Qt::Checked);
                return;
            }
        } else if (!plugin_info.is_loaded && item->checkState(FIELD_IS_LOADED) == Qt::Checked) {
            if (!glue->loadPlugin(plugin_name, errmsg)) {
                qWarning("%s", (tr("Unable to load plugin '%1': %2").arg(plugin_name).arg(errmsg)).toUtf8().data());
                item->setCheckState(FIELD_IS_LOADED, Qt::Unchecked);
                return;
            }
        }
    }

    if (column == FIELD_AUTO_LOAD) {
        QStringList auto_load_plugins = gui_opts.getAutoLoadPlugins();

        if (item->checkState(FIELD_AUTO_LOAD) == Qt::Checked) {
            auto_load_plugins.append(plugin_name);
        } else {
            auto_load_plugins.removeOne(plugin_name);
        }

        gui_opts.setAutoLoadPlugins(auto_load_plugins);
    }
}

void DialogManagePlugins::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
