#include "WidgetConfigServers.h"
#include "ui_WidgetConfigServers.h"
#include "DialogAddServer.h"
#include <nntpgrab_utils.h>

WidgetConfigServers::WidgetConfigServers(QNNTPGrabGlue *glue, QWidget *parent) :
        QWidget(parent),
        ui(new Ui::WidgetConfigServers)
{
    this->glue = glue;

    ui->setupUi(this);

    /* Server list */
    ui->treeWidget->header()->setResizeMode(QHeaderView::ResizeToContents);

    onConfigChanged();

    connect(ui->chkEnableBandwidthLimiting, SIGNAL(toggled(bool)), SLOT(chkEnableBandwidthLimiting_toggled(bool)));
    connect(ui->btnApplyBandwidthLimiting, SIGNAL(clicked()), SLOT(on_btnApplyBandwidthLimiting_clicked()));
    connect(glue, SIGNAL(configChanged()), SLOT(onConfigChanged()));
}

WidgetConfigServers::~WidgetConfigServers()
{
    delete ui;
}

void WidgetConfigServers::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfigServers::chkEnableBandwidthLimiting_toggled(bool toggled)
{
    if (toggled) {
        ui->spinMaxBandwidth->show();
        ui->btnApplyBandwidthLimiting->show();
    } else {
        ui->spinMaxBandwidth->hide();
        ui->btnApplyBandwidthLimiting->hide();
    }

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (opts.enable_bandwidth_shaping == toggled) {
        return;
    }

    opts.enable_bandwidth_shaping = toggled;
    opts.max_bandwidth = ui->spinMaxBandwidth->value();
    glue->configSetOpts(opts);
}

void WidgetConfigServers::on_btnApplyBandwidthLimiting_clicked()
{
    qDebug("btn ApplyBandwidthLimiting clicked");
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    opts.max_bandwidth = ui->spinMaxBandwidth->value();
    glue->configSetOpts(opts);
}

void WidgetConfigServers::on_btnAddServer_clicked()
{
    qDebug("btn Addserver clicked");
    DialogAddServer dialog(glue);
    dialog.exec();
}

void WidgetConfigServers::UpdateServers()
{
    QList<QString> servers = glue->configGetAvailServers();
    for (int i = 0; i < servers.length(); i++) {
        QNNTPGrabGlue::ConfigServer server;

        if (!glue->configGetServerInfo(servers.at(i), server)) {
            /* Shouldn't happen */
            qDebug("configGetServerInfo returned FALSE");
            continue;
        }

        QTreeWidgetItem *server_item = new QTreeWidgetItem();
        server_item->setText(0, server.servername);
        switch (server.priority) {
        case SERVER_PRIORITY_HIGH:
            server_item->setText(1, tr("High"));
            break;
        case SERVER_PRIORITY_LOW:
            server_item->setText(1, tr("Low"));
            break;
        case SERVER_PRIORITY_NORMAL:
        default:
            server_item->setText(1, tr("Normal"));
            break;
        }
        server_item->setText(2, QString("%1").arg(server.max_threads));
        if (server.enabled) {
            server_item->setCheckState(3, Qt::Checked);
        } else {
            server_item->setCheckState(3, Qt::Unchecked);
        }

        ui->treeWidget->addTopLevelItem(server_item);
    }
}

void WidgetConfigServers::on_btnRemoveServer_clicked()
{
    QString errmsg = "";
    if(ui->treeWidget->selectedItems().count() == 1) {
        qDebug("remove server %s", ui->treeWidget->selectedItems()[0]->text(0).toUtf8().data());
        if (!glue->configDelServer(ui->treeWidget->selectedItems()[0]->text(0), errmsg)) {
            qWarning("%s", errmsg.toUtf8().data());
        }
    }
}

void WidgetConfigServers::on_btnEditServer_clicked()
{
    if(ui->treeWidget->selectedItems().count() == 1) {
        qDebug("edit server %s", ui->treeWidget->selectedItems()[0]->text(0).toUtf8().data());
        QNNTPGrabGlue::ConfigServer server;

        if (!glue->configGetServerInfo(ui->treeWidget->selectedItems()[0]->text(0), server)) {
            qDebug("configGetServerInfo failed");
            return;
        }

        DialogAddServer dialog(glue, &server);
        dialog.exec();
    }
}

void WidgetConfigServers::onConfigChanged()
{
    /* Rebuild list of servers */
    ui->treeWidget->clear();
    this->UpdateServers();

    /* Bandwidth limiting */
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    ui->chkEnableBandwidthLimiting->setChecked(opts.enable_bandwidth_shaping);
    ui->spinMaxBandwidth->setValue(opts.max_bandwidth);
    chkEnableBandwidthLimiting_toggled(opts.enable_bandwidth_shaping);
}

void WidgetConfigServers::on_treeWidget_itemSelectionChanged()
{
    if(ui->treeWidget->selectedItems().count() == 1) {
        ui->btnEditServer->setEnabled(true);
        ui->btnRemoveServer->setEnabled(true);
    } else {
        ui->btnEditServer->setEnabled(false);
        ui->btnRemoveServer->setEnabled(false);
    }

}

void WidgetConfigServers::on_treeWidget_itemDoubleClicked(QTreeWidgetItem* item, int column)
{
    Q_UNUSED(column);
    QNNTPGrabGlue::ConfigServer server;

    if (!glue->configGetServerInfo(item->text(0), server)) {
        qDebug("configGetServerInfo failed");
        return;
    }
    DialogAddServer dialog(glue, &server);
    dialog.exec();
}

void WidgetConfigServers::on_treeWidget_itemClicked(QTreeWidgetItem* item, int column)
{
    QString errmsg = "";

    if(column ==  3) {
        QNNTPGrabGlue::ConfigServer server;

        if (!glue->configGetServerInfo(item->text(0), server)) {
            qDebug("configGetServerInfo failed");
            return;
        }
        if(item->checkState(3) == Qt::Checked) {
            server.enabled = true;
        } else {
            server.enabled = false;
        }
        if(!glue->configEditServer(QString(server.servername), server, errmsg)) {
            qWarning("%s", errmsg.toUtf8().data());
            return;
        }
    }
}
