#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QSystemTrayIcon>

#include "NGApplication.h"
#include <QNNTPGrabGlue.h>
#include "ConfigGuiOpts.h"
#include "WidgetUpdateInfo.h"
#include "WidgetConfig.h"
#include "WidgetDownloadQueue.h"
#include "WidgetNZBCreator.h"
#include "WidgetRepairAndUnpack.h"
#include "WidgetTrafficMonitor.h"
#include "WidgetConnections.h"

#include "FormDebug.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QNNTPGrabGlue *glue, NGApplication *app, QWidget *parent = 0);
    ~MainWindow();

    void setShowNotificationOnFileDownloaded(bool enabled);
    void setShowNotificationOnFileDecoded(bool enabled);
    void showNotification(QString title, QString msg);
    void showTrayIcon();
    void hideTrayIcon();

protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent *event);

#ifdef Q_WS_WIN
    bool winEvent(MSG* msg, long* result);
#elif defined(Q_WS_X11)
    void hideEvent(QHideEvent *e);
#endif

private:
    Ui::MainWindow *ui;
    QNNTPGrabGlue *glue;
    ConfigGuiOpts gui_opts;

    bool trayIconIsShown;
    QSystemTrayIcon trayIcon;

    bool showNotificationOnFileDownloaded;
    bool showNotificationOnFileDecoded;

    WidgetUpdateInfo *updateInfoWidget;
    WidgetConfig *configWidget;
    WidgetDownloadQueue *downloadQueueWidget;
    WidgetNZBCreator *nzbcreatorWidget;
    WidgetRepairAndUnpack *repairAndUnpackWidget;

    WidgetTrafficMonitor *trafficMonitorWidget;
    WidgetConnections *connectionsWidget;

    FormDebug *formDebug;

private slots:
    void actionImportNZB_activated(void);
    void actionDonate_activated(void);
    void actionEnableAutomaticShutdown_activated(void);
    void actionKillNNTPGrabServer_activated(void);
    void actionManagePlugins_activated(void);
    void actionShowDebugWindow_activated(void);
    void actionPauseDownloadQueue_activated(void);

    void onFileStateChanged(QString collection_name, QString subject, QString real_filename, QNNTPGrabGlue::TaskState old_state, QNNTPGrabGlue::TaskState new_state, quint64 file_size_remaining, quint64 total_size, quint64 total_size_remaining);
    void onSchedularStateChanged(QNNTPGrabGlue::SchedularState state, QString reason);
    void onConnectionLost(QString errmsg);
    void onAllDownloadsCompleted(void);

    void trayIconActivated(QSystemTrayIcon::ActivationReason);

    void onMessageReceived(QString);

    void loadNZBFile(QString filename);
};

#endif // MAINWINDOW_H
