/*
    Copyright (C) 2005-2012  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_GLUE_INTERNAL_H_
#define _NNTPGRAB_GLUE_INTERNAL_H_

#include <glib.h>
#include <glib-object.h>
#include "nntpgrab_glue.h"
#include "nntpgrab.h"

#ifdef  __cplusplus
extern "C" {
#endif

#define READ_TIMEOUT_VALUE      30

enum
{
    CONFIG_CHANGED_SIGNAL,
    PART_DOWNLOAD_START_SIGNAL,
    PART_DONE_SIGNAL,
    PART_FAILED_SIGNAL,
    PART_PROGRESS_UPDATE_SIGNAL,
    TRAFFIC_MONITOR_UPDATE_SIGNAL,
    COLLECTION_ADDED_SIGNAL,
    COLLECTION_REMOVED_SIGNAL,
    COLLECTION_MODIFIED_SIGNAL,
    FILE_ADDED_SIGNAL,
    FILE_REMOVED_SIGNAL,
    FILE_DOWNLOAD_STATE_UPDATE_SIGNAL,
    FILE_STATE_CHANGED_SIGNAL,
    CONNECTION_CONNECTING_SIGNAL,
    CONNECTION_CONNECTED_SIGNAL,
    CONNECTION_DISCONNECT_SIGNAL,
    SCHEDULAR_STATE_CHANGED_SIGNAL,
    LOG_MESSAGE_SIGNAL,
    CONNECTION_LOST_SIGNAL,
    TASK_MOVED_SIGNAL,
    COLLECTION_MOVED_SIGNAL,
    COLLECTION_DOWNLOADED_SIGNAL,
    ALL_DOWNLOADS_COMPLETED_SIGNAL,
    PLUGIN_LOADED_SIGNAL,
    PLUGIN_UNLOADED_SIGNAL,
    PLUGIN_EVENT_SIGNAL,
    LAST_SIGNAL
};

typedef struct _connection {
    int socket_id;
    GString *recv_buf;
} Connection;

struct NntpgrabGlue
{
    GObject parent;

    Connection socket;

    int glue_version;

    gboolean is_initialized;

    NntpgrabCore *standalone_core;
};

gboolean write_line(Connection *sock, const char *format, ...);
gboolean has_signal_handler_pending(int signal_id);

void     connect_signal_handlers(NntpgrabCore *core);
void     disconnect_signal_handlers(NntpgrabCore *core);

void     glue_process_jsonrpc_msg(const char *msg);

void             nntpgrab_glue_internal_kill_server(NntpgrabGlue *obj);
NGList          *nntpgrab_glue_internal_config_get_avail_servers(NntpgrabGlue *obj);
gboolean         nntpgrab_glue_internal_config_get_server_info(NntpgrabGlue *obj, const char *servername, NGConfigServer *ret);
gboolean         nntpgrab_glue_internal_config_add_server(NntpgrabGlue *obj, NGConfigServer new_server, char **errmsg);
gboolean         nntpgrab_glue_internal_config_del_server(NntpgrabGlue *obj, const char *servername, char **errmsg);
gboolean         nntpgrab_glue_internal_config_edit_server(NntpgrabGlue *obj, const char *servername, NGConfigServer new_server, char **errmsg);
gboolean         nntpgrab_glue_internal_config_get_opts(NntpgrabGlue *obj, NGConfigOpts *opts);
gboolean         nntpgrab_glue_internal_config_set_opts(NntpgrabGlue *obj, NGConfigOpts opts);
gboolean         nntpgrab_glue_internal_config_get_folder_listing(NntpgrabGlue *obj, const char *parent, NGList **folders);
gboolean         nntpgrab_glue_internal_schedular_start(NntpgrabGlue *obj);
gboolean         nntpgrab_glue_internal_schedular_stop(NntpgrabGlue *obj, gboolean wait);
NGSchedularState nntpgrab_glue_internal_schedular_get_state(NntpgrabGlue *obj);
gboolean         nntpgrab_glue_internal_schedular_add_task_to_queue(NntpgrabGlue *obj, const char *collection_name, const char *subject, const char *poster, time_t stamp, nguint64 file_size, NGList *groups, NGList *parts, char **errmsg);
gboolean         nntpgrab_glue_internal_schedular_del_task_from_queue(NntpgrabGlue *obj, const char *collection_name, const char *subject, char **errmsg);
gboolean         nntpgrab_glue_internal_schedular_restart_task(NntpgrabGlue *obj, const char *collection_name, const char *subject, char **errmsg);
gboolean         nntpgrab_glue_internal_schedular_save_queue(NntpgrabGlue *obj, char **errmsg);
gboolean         nntpgrab_glue_internal_schedular_foreach_task(NntpgrabGlue *obj, ForeachCollectionFunc collection_func, ForeachFileFunc file_func, ForeachGroupFunc group_func, void *data);
gboolean         nntpgrab_glue_internal_schedular_move_task(NntpgrabGlue *obj, const char *collection_name_src, const char *subject_src, const char *collection_name_dest, int position_dest);
gboolean         nntpgrab_glue_internal_schedular_move_collection(NntpgrabGlue *obj, const char *collection_name, int new_position);
gboolean         nntpgrab_glue_internal_schedular_mark_task_optional(NntpgrabGlue *obj, const char *collection_name, const char *subject, ngboolean is_optional);
NGList          *nntpgrab_glue_internal_plugins_get_avail_plugins(NntpgrabGlue *obj);
gboolean         nntpgrab_glue_internal_plugins_get_plugin_info(NntpgrabGlue *obj, const char *plugin_name, NNTPGrabPluginInfo *plugin_info);
gboolean         nntpgrab_glue_internal_plugins_load_plugin(NntpgrabGlue *obj, const char *plugin_name, char **errmsg);
gboolean         nntpgrab_glue_internal_plugins_unload_plugin(NntpgrabGlue *obj, const char *plugin_name, char **errmsg);
gboolean         nntpgrab_glue_internal_plugins_set_persistent(NntpgrabGlue *obj, const char *plugin_name, ngboolean persistent);
void             nntpgrab_glue_internal_set_emit_log_messages(NntpgrabGlue *obj, ngboolean val);

#ifdef  __cplusplus
}
#endif

#endif /* _NNTPGRAB_GLUE_H_ */
